#include <pic.h>

#define set_scl RC3=1		/*SCL pin HIGH definition. Change to your own pin! */
#define clr_scl RC3=0		/*SCL pin LOW definition. Change to your own pin! */
#define set_sda TRISC4=1	/*SDA pin HIGH definition. Change to your own pin!
				  pin must be set to INPUT via TRIS because I2C data line is open 
				  collector */
#define clr_sda	{TRISC4=0; RC4=0;}	/*SDA pin HIGH definition.
				  This macro only sets the SDA pin to an output, you still have to 
				  manually ensure that the SDA pin PORT register is set LOW 
				  This is doen by running the init_I2C routine */
#define sda_data RC4		/*the SDA input data. Change to your own pin! */
#define high=1			/*HIGH bus state definition */
#define low=0			/*LOW bus state definition */

/* *******************************************************************
	This routine must be run at least once in your code before
	any of the I2C routines below. No need to run it more than once
	unless your other PIC code is changing your pins! */
void init_I2C() {
	RC4=0;		/* set the SDA pin LOW. The SDA pin is then set 
			HIGH by the TRIS command */
	TRISC3=0;	/* set SCL line to an OUPUT */
	TRISC4=1;	/* set SDA line to an INPUT just to be sure! */
}
	
/* *******************************************************************
	The "start" routine sets the I2C bus to the START condition */
void I2C_start() {
	set_scl;
	set_sda;
	clr_sda;
	clr_scl;		
}

/* *******************************************************************
	The "stop" routine sets the I2C bus to the STOP condition */
void I2C_stop() {
	clr_sda;
	set_scl;
	set_sda;
	set_scl;		
}

/* *******************************************************************
	The "stable" routine sets the I2C bus to the STABLE condition */
void I2C_stable() {
	set_sda;
	set_scl;
}

void I2C_ack() {
  clr_sda;
  set_scl;
  clr_scl;
  set_sda;
}

void I2C_nac() {
  set_sda;
  set_scl;
  clr_scl;
}

/* *******************************************************************
	"sendbyte" sends a byte down the I2C bus
	The byte to be sent is passed to the routine via "sendbuf" */
unsigned char I2C_sendbyte(unsigned char sendbuf) {

unsigned char t;

	clr_scl;			/* set scl low initially */

	/* note:I2C requires MSBit to be sent first */
	
	if ((sendbuf&128)==0) clr_sda else set_sda;	/* ouput bit7 */
	set_scl;				/* toggle clock line */
	clr_scl;

	if ((sendbuf&64)==0) clr_sda else set_sda;	/* ouput bit6 */
	set_scl;				/* toggle clock line */
	clr_scl;

	if ((sendbuf&32)==0) clr_sda else set_sda;	/* ouput bit5 */
	set_scl;				/* toggle clock line */
	clr_scl;

	if ((sendbuf&16)==0) clr_sda else set_sda;	/* ouput bit4 */
	set_scl;				/* toggle clock line */
	clr_scl;

	if ((sendbuf&8)==0) clr_sda else set_sda;	/* ouput bit3 */
	set_scl;				/* toggle clock line */
	clr_scl;

	if ((sendbuf&4)==0) clr_sda else set_sda;	/* ouput bit2 */
	set_scl;				/* toggle clock line */
	clr_scl;

	if ((sendbuf&2)==0) clr_sda else set_sda;	/* ouput bit1 */
	set_scl;				/* toggle clock line */
	clr_scl;

	if ((sendbuf&1)==0) clr_sda else set_sda;	/* ouput bit0 */
	set_scl;				/* toggle clock line */
	clr_scl;
	
	set_sda;
	set_scl;	
	t = 1-sda_data;
	clr_scl;
	return t;
}

/* *******************************************************************
	"readbyte" reads a byte from the I2C bus 
	Function returns the byte received
	NOTE: This routine has no timeout, so may get stuck in an infinite
	loop if it does not get the correct data from the I2C port */
unsigned char I2C_readbyte() {
	unsigned char byteread;
	byteread=0;	/* assume the data will be 0 */
	clr_scl;
	set_sda;	/*set SDA line to INPUT */

	set_scl;
	if (sda_data==1) byteread+=128;		/* get bit7 */
	clr_scl;

	set_scl;
	if (sda_data==1) byteread+=64;		/* get bit6 */
	clr_scl;

	set_scl;
	if (sda_data==1) byteread+=32;		/* get bit5 */
	clr_scl;

	set_scl;
	if (sda_data==1) byteread+=16;		/* get bit4 */
	clr_scl;

	set_scl;
	if (sda_data==1) byteread+=8;		/* get bit3 */
	clr_scl;

	set_scl;
	if (sda_data==1) byteread+=4;		/* get bit2 */
	clr_scl;

	set_scl;
	if (sda_data==1) byteread+=2;		/* get bit1 */
	clr_scl;

	set_scl;
	if (sda_data==1) byteread+=1;		/* get bit0 */
	clr_scl;
		
	return byteread;	/* return the byte read from the bus */
}

void I2C_Command(unsigned char device, unsigned char command) {
  I2C_stable();
  I2C_start();
  I2C_sendbyte(device & 0b11111110);
  I2C_sendbyte(command);
  I2C_stop();
}

unsigned char I2C_Read(unsigned char device, unsigned char addr) {
unsigned char data;
  I2C_stable();
  I2C_start();
  I2C_sendbyte(device & 0b11111110);
  I2C_sendbyte(addr);
//  I2C_stop();	//	- continious start
  I2C_start();
  I2C_sendbyte(device | 0b00000001);
  data = I2C_readbyte();
  I2C_nac();
  I2C_stop();
  return data;
}

unsigned int I2C_ReadWord(unsigned char device, unsigned char addr) {
unsigned int data;
  I2C_stable();
  I2C_start();
  I2C_sendbyte(device & 0b11111110);
  I2C_sendbyte(addr);
//  I2C_stop();	//	-	    
  I2C_start();
  I2C_sendbyte(device | 0b00000001);
  data = I2C_readbyte() << 8;
  I2C_ack();
  data = data |I2C_readbyte();
  I2C_nac();
  I2C_stop();
  return data;
}

void I2C_Write(unsigned char device, unsigned char addr, unsigned char data) {
  I2C_stable();
  I2C_start();
  I2C_sendbyte(device & 0b11111110);
  I2C_sendbyte(addr);
  I2C_sendbyte(data);
  I2C_stop();
}

void I2C_WriteWord(unsigned char device, unsigned char addr, unsigned int data) {
  I2C_stable();
  I2C_start();
  I2C_sendbyte(device & 0b11111110);
  I2C_sendbyte(addr);
  I2C_sendbyte(data >> 8);
  I2C_sendbyte(data & 0x00FF);
  I2C_stop();
}

