        
#define TMR1Sconst 976
#define BlinkConst 0x400
#define ProgConst 8
#include <pic.h>
#include "always.h"
#include "i2c.inc"

unsigned int  tu=0;
unsigned char ts=0;
unsigned char tw=0;

unsigned char const  digits [16] = {
  0b10100000, // 0
  0b10111110, // 1
  0b01100010, // 2
  0b00101010, // 3
  0b00111100, // 4
  0b00101001, // 5
  0b00100001, // 6
  0b10111010, // 7
  0b00100000, // 8
  0b00101000, // 9
  0b00110000, // A
  0b00100101, // b
  0b11100001, // C
  0b00100110, // d
  0b01100001, // E
  0b00110001  // F
};

unsigned char tdisp=1, tblink=0x00;

unsigned char d0=0b11111111,
              d1=0b11111111,
              d2=0b11111111,
              d3=0b11111111;

bit d0b=0, d1b=0, d2b=0, d3b=0;


bit fs = 0;

bit writing = 0;

signed int tmp1 = 0x7F00;
signed int tmp2 = 0x7F00;

void interrupt timer(void) {
  
  if (T0IF && T0IE) {
    T0IF = 0;
    tu++;
    if (tu == TMR1Sconst) {
      tu = 0;
      ts++;
      fs = 1 - fs;
    }

    if (((tu & 0x0003) == 0)&& !writing) {
      tw++;
      tdisp++;
      if (tdisp >= 4)
        tdisp = 0;
//      PORTB = 0xFF;
      PORTA &= 0b11110000;
      switch (tdisp) {
        case 0: RA1 = 1; if (!d0b || ((tu & BlinkConst)!=0)) PORTB = d0; break;
        case 1: RA2 = 1; if (!d1b || ((tu & BlinkConst)!=0)) PORTB = d1; break;
        case 2: RA3 = 1; if (!d2b || ((tu & BlinkConst)!=0)) PORTB = d2; break;
        case 3: RA0 = 1; if (!d3b || ((tu & BlinkConst)!=0)) PORTB = d3; break;
      }
    }
  }
}

void init(void) {
  STATUS = 0;
  TRISA  = 0b00010000;
  TRISB  = 0b00000000;
  TRISC  = 0b00000100;
  PORTA  = 0;
  PORTB  = 0;
  PORTC  = 0;
  OPTION = 0b10001111;
  INTCON = 0;
  ADCON1 = 0b00000110;
  SSPCON=0;
  T0IF = 0;
  T0IE = 1;
  GIE = 1;

  init_I2C();

  I2C_Command(0b10010000, 0x22);
  I2C_Write(0b10010000, 0xAC, 0b10011001);
  I2C_Command(0b10010000, 0xEE);

  I2C_Command(0b10010010, 0x22);
  I2C_Write(0b10010010, 0xAC, 0b10011001);
  I2C_Command(0b10010010, 0xEE);

}

void drawt(signed int value) {
  writing = 1;
  if (value == 0x7F00) {
    d0 = 0xFF;
    d1 = 0x61;  //E
    d2 = 0x77;  //r
    d3 = 0x77;  //r
  } else if (value>0) {
    d0 = digits[((value>>8) / 100) % 10];
    d1 = digits[((value>>8) / 10) % 10];
    d2 = digits[(value>>8) % 10];
    d2 &= 0xDF;
    if (value & 0x00FF)
      d3 = digits[5];
    else
      d3 = digits[0];
  } else {
    d0 = 0x7F;
    d1 = digits[(((-value)>>8) / 10) % 10];
    d2 = digits[((-value)>>8) % 10];
    d2 &= 0xDF;
    if (value & 0x00FF)
      d3 = digits[5];
    else
      d3 = digits[0];
  }
  writing = 0;
}

unsigned char dconfig;

void main(void) {
  init();
  drawt(0x7F00);
  while (1) {
    if (tw >= ProgConst) {
        tw = 0;
        asm("clrwdt");
      if (fs) {
      	tmp1 = 0x7F00;
        dconfig = I2C_Read(0b10010000, 0xAC);
        if (testbit_on(dconfig, 2) || !testbit_on(dconfig, 3)) {
          I2C_Command(0b10010000, 0x22);
          I2C_Write(0b10010000, 0xAC, dconfig | 0x01);
          I2C_Command(0b10010000, 0xEE);
        } else
          if (testbit_on(dconfig, 7)) {
            tmp1 = I2C_ReadWord(0b10010000, 0xAA);
            I2C_Command(0b10010000, 0xEE);
         }
         drawt(tmp1);
      } else {
      	tmp2 = 0x7F00;
        dconfig = I2C_Read(0b10010010, 0xAC);
        if (testbit_on(dconfig, 2) || !testbit_on(dconfig, 3)) {
          I2C_Command(0b10010010, 0x22);
          I2C_Write(0b10010010, 0xAC, dconfig | 0x01);
          I2C_Command(0b10010010, 0xEE);
        } else
          if (testbit_on(dconfig, 7)) {
            tmp2 = I2C_ReadWord(0b10010010, 0xAA);
            I2C_Command(0b10010010, 0xEE);
         }
         drawt(tmp2);
      }

    }
  }  
}

